#ifndef __NETWORK_CONFIG_HH__
#define __NETWORK_CONFIG_HH__

#include "kvtree.hh"

namespace kvtree {
  namespace node_types {

    /**************** interface keywords *********************/
    const node_type interface_section = 1;


    const node_type address_subsection = 20;
    // Not really :  const node_type dhcp = 201;
    const node_type ip_setup = 202;
    const node_type gw_setup = 203;
    const node_type search = 204;
    const node_type nameserver = 205;


    const node_type firewall_subsection = 30;
    const node_type accept = 300;
    const node_type reject = 301;
    const node_type drop = 302;


    const node_type wireless_subsection = 40;
    const node_type wireless_mode = 401;
    const node_type wireless_channel = 402;
    const node_type wireless_essid = 403;
    const node_type wireless_key = 404;


    const node_type ppp_subsection = 50;
    const node_type ppp_device = 501;
    // Not really: const node_type ppp_pppoe = 502;
    // Not really: const node_type ppp_defaults = 503;
    // Not really: const node_type ppp_chat_defaults = 504;
    const node_type ppp_chat_init = 505;
    const node_type ppp_chat_dial = 506;
    const node_type ppp_option = 507;
    // Not really: const node_type ppp_speed_defaults = 508;


    const node_type masquerade = 90;

    /***************** global keywords ****************/

    // in this preliminary version, comments are not stored,
    // so followning is a useless tag for now.
    const node_type comment = 2;
    
    // TODO: the remaining global options
  }
}

namespace network_config {

  typedef kvtree::node_type node_type;

  class IP
  {
  public:
    typedef unsigned char block;
    block address[4];

    IP ();
    void SetDefault (); // 127.0.0.1
    void SetTo (block a, block b, block c, block d) { // coder friendly version to set a fixed ip
      address[0]=a; address[1]=b; address[2]=c; address[3]=d;
    }


    bool FromString (const std::string& str);
    std::string ToString ();
  };

  class CIDR : public IP
  {
  public:
    int netmask_bits;

    CIDR ();

    bool FromString (const std::string& str);
    std::string ToString ();
  };

  
  /*************************** Interface section ****************************************/
 
  class AddressSubsection;
  class WirelessSubsection;
  class PPPSubsection;

  class InterfaceSection : public kvtree::Group
  {
  public:
    std::string interface;
    std::string profile;

    InterfaceSection ( const std::string& iface, const std::string& prof, kvtree::Group* root);

    // why did i want these functions again ?
    //bool skipWireless () { return skip_wireless; }
    //void setSkipWireless (bool state) { skip_wireless = state; }

    bool Enabled () {return enabled;}
    void setEnabled (bool state) {enabled = state;}
    
    AddressSubsection* GetAddressSubsection (bool autocreate = true);
    WirelessSubsection* GetWirelessSubsection (bool autocreate = true);
    PPPSubsection* GetPPPSubsection (bool autocreate = true);

    
    virtual node_type Type () { return kvtree::node_types::interface_section; }

    // dumps all sections beside the ones that are disabled
    virtual void Dump (std::ostream& stream);
  private:
    //    bool skip_wireless;
    bool enabled;
  };



  /*************************** Address specific part ***********************************/

  class IPBaseLine : public kvtree::Line
  {
  public:
    virtual IP* GetIP () = 0;
    virtual void Set () = 0;  // invoked by user programm when address changed
    IPBaseLine (const std::string& line) : kvtree::Line (line) {};
  };

  class IPLine : public IPBaseLine
  {
  public:
    virtual node_type Type () { return kvtree::node_types::ip_setup; }
    CIDR address;    

    IPLine (const std::string& line) : IPBaseLine (line) {};

    virtual IP* GetIP () { return &address; }
    virtual void Set (); // invoked by user programm when address changed
  };

  // HACK wrapper for old-style configs - not about to be inserted as a child !
  class NetmaskLine : public IPBaseLine
  {
  public:
    virtual node_type Type () { return kvtree::node_types::unknown; }
    IPLine* ipl;
    IP faked;

    NetmaskLine (IPLine* origin) : IPBaseLine (origin -> line) {ipl = origin;}

    virtual IP* GetIP ();
    virtual void Set ();
  };

  class GatewayLine : public IPBaseLine
  {
  public:
    virtual node_type Type () { return kvtree::node_types::gw_setup; }
    IP nexthop;
    // TODO: metric
   
    GatewayLine (const std::string& line) : IPBaseLine (line) {};

    virtual IP* GetIP () { return &nexthop; }   
    virtual void Set (); // invoked by user programm when parameters changed
  };

  class SearchLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::search; }
    std::string domain;
  
    SearchLine (const std::string& line) : kvtree::Line (line) {};
 
    void Set (); // invoked by user programm when parameters changed
  };

  class NameserverLine : public IPBaseLine
  {
  public:
    virtual node_type Type () { return kvtree::node_types::nameserver; }
    IP nameserver;

    NameserverLine (const std::string& line) : IPBaseLine (line) {};

    virtual IP* GetIP () { return &nameserver; }
    virtual void Set (); // invoked by user programm when parameters changed
  };


  class AddressSubsection : public kvtree::Group
  {
  public:
    AddressSubsection ( kvtree::Group* interface);

    // following functions return 0 if the line that was asked for does not exist
    IPLine* Address (unsigned int number=0);
    GatewayLine* Gateway (unsigned int number=0);
    SearchLine* Search (unsigned int number=0);
    NameserverLine* Nameserver (unsigned int number=0);

    bool Dhcp () { return dhcp; }
    bool skipNameservers () { return skip_nameservers; }
    bool skipGateways () { return skip_gateways; }

    void setDhcp (bool state) { dhcp = state; }
    void setSkipNameservers (bool state) { skip_nameservers = state; }
    void setSkipGateways (bool state) { skip_gateways = state;}


    virtual node_type Type () { return kvtree::node_types::address_subsection; }

    // when dump is invoked, either children (w. or wo. gw/nameserver)
    // are dumped or generic dhcp line is written

    virtual void Dump (std::ostream& stream);



  private:
    bool dhcp;

    bool skip_nameservers;
    bool skip_gateways;
  };


  /*************************** Wireless specific part ***********************************/

  class ModeLine : public kvtree::Line
  {
  public:
    typedef enum {
      managed,
      adhoc
    } operating_mode;

    virtual node_type Type () { return kvtree::node_types::wireless_mode; }
    
    operating_mode mode;
  
    ModeLine (const std::string& line) : kvtree::Line (line) { mode = managed; }
 
    void Set (); // invoked by user programm when parameters changed
    bool FromString (const std::string& str);
  };

  class ChannelLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::wireless_channel; }
    unsigned int channel;

    ChannelLine (const std::string& line) : kvtree::Line (line) { channel = 1; }
 
    void Set (); // invoked by user programm when parameters changed
    bool FromString (const std::string& str);
  };

  class EssidLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::wireless_essid; }
    std::string essid; // empty string is "any"

    EssidLine (const std::string& line) : kvtree::Line (line) {};
 
    void Set (); // invoked by user programm when parameters changed
  };

  class KeyLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::wireless_key; }
    std::string key; // empty string is "off"

    KeyLine (const std::string& line) : kvtree::Line (line) {};
 
    void Set (); // invoked by user programm when parameters changed
  };


  class WirelessSubsection : public kvtree::Group
  {
  public:
    WirelessSubsection ( kvtree::Group* interface);

    virtual node_type Type () { return kvtree::node_types::wireless_subsection; }    

    // Features automatic creation
    ModeLine* Mode ();
    ChannelLine* Channel ();
    EssidLine* Essid ();
    KeyLine* Key ();

    // dumps all children but ignores channel line when mode=managed
    virtual void Dump  (std::ostream& stream);
  };


  
  class DeviceLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::ppp_device; }
    std::string device;
    unsigned int speed; // set to 0 for default

    DeviceLine (const std::string& line) : kvtree::Line (line) { speed=0; }
 
    void Set (); // invoked by user programm when parameters changed
    bool SpeedFromString (const std::string& speed_str);
  };


  class ChatInitLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::ppp_chat_init; }
    std::string init;

    ChatInitLine (const std::string& line) : kvtree::Line (line) {};
 
    void Set (); // invoked by user programm when parameters changed
  };

  class ChatDialLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::ppp_chat_dial; }
    std::string dial;

    ChatDialLine (const std::string& line) : kvtree::Line (line) {};
 
    void Set (); // invoked by user programm when parameters changed
  };


  class OptionLine : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::ppp_option; }
    std::string option;
    std::string value;

    OptionLine (const std::string& line) : kvtree::Line (line) {};
 
    void Set (); // invoked by user programm when parameters changed
  };



  class PPPSubsection : public kvtree::Group
  {
  public:
    PPPSubsection ( kvtree::Group* interface);

    virtual node_type Type () { return kvtree::node_types::ppp_subsection; }

    OptionLine* Option (unsigned int number);
    OptionLine* Option (const std::string& name, bool autocreate=false);
    DeviceLine* Device ();
    ChatInitLine* ChatInit ();
    ChatDialLine* ChatDial ();


    bool OverEthernet () { return pppoe; }
    bool Defaults () { return ppp_defaults; }
    bool SpeedDefaults () { return ppp_speed_defaults; }
    bool ChatDefaults () { return ppp_chat_defaults; }

    void SetOverEthernet (bool state) { pppoe = state; }
    void SetDefaults (bool state) { ppp_defaults = state; }
    void SetSpeedDefaults (bool state) { ppp_speed_defaults = state; }
    void SetChatDefaults (bool state) { ppp_chat_defaults = state; }

    // dumps all except empty/disabled user/passwd
    // and skips chat stuff in pppoe mode
    virtual void Dump (std::ostream& stream);    
  private:
    bool pppoe;
    bool ppp_defaults;
    bool ppp_speed_defaults;
    bool ppp_chat_defaults;
  };


/* skeleton
  class Line : public kvtree::Line
  {
  public:
    virtual node_type Type () { return kvtree::node_types::; }
  
    Line (const std::string& line) : kvtree::Line (line) {};
 
    void Set (); // invoked by user programm when parameters changed
  };
  */




}

#endif
