#include <qlabel.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qlayout.h>
#include <klocale.h>
#include <qpushbutton.h>

#include "modem_status.hh"
#include "systeminfo.hh"
#include "pixmaps.hh"
#include "profile_manager.hh"
#include "ppp_status.hh"

ModemStatus::ModemStatus (QWidget* parent, ProfileManager* manager, const std::string& device)
  : QHBox (parent)
{
  dev = device;
  m_manager = manager;
  online = ::GetSmallTelephone (false);
  offline = ::GetSmallTelephone (true);

  button = new QPushButton (this);
  button -> setMaximumWidth (32);
  button -> setPixmap (*offline);

  status = new QLabel (this);
  QHBoxLayout* l = (QHBoxLayout*) layout ();
  l ->   insertStretch (2, 0);
  m_timer = new QTimer (this);
  QObject::connect( m_timer, SIGNAL(timeout()), this, SLOT(RefreshStatus()) );
  QObject::connect( button, SIGNAL(clicked()), this, SLOT(ButtonPushed()) );
  my_status = new PPPStatus (std::string("/var/log/") + device);
  is_up = false; // info
}


void ModemStatus::SetActive (bool state)
{
  if (state) {
    status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("offline.") );
    if (m_manager -> interfaceIsUp (dev) && !is_up) {
      is_up = true;
      TakeUp ();
    }
  } else {
    if (is_up)
      TakeDown ();
  }
}


void ModemStatus::RefreshStatus ()
{
  ppp_state state = my_status -> GetStatus ();

  switch (state) {
  case ppp_states::terminated:
    switch (my_status -> GetError ()) {
    case ppp_errors::no_dialtone:
      status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("no dialtone.\nMake sure the modem is connected.") );
      break;
    case ppp_errors::busy:
      status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("line busy.\nTry later.") );
      break;
    case ppp_errors::timeout:
      status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("no response from modem.\nMake sure modem is connected.") );
      break;
    case ppp_errors::authentification:
      status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("authentification failed.\nPlease check password/username.") );
      break;
    case ppp_errors::hangup:
      status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("reset by peer.\nTry again.") );
      break;
    case ppp_errors::unknown:
      status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("connection error.\nAn unknown error has occured.") );
      break;
    case ppp_errors::no_error:
    default:
      status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("terminated.") );
    }
    if (m_timer -> isActive ())
      m_timer -> stop ();
    break;
  case ppp_states::connecting:
    status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("connecting...") );
    break;
  case ppp_states::connected:
    status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("online:") + QString (" ") + TimeToString ());
    break;
  case ppp_states::no_connection:
    status -> setText (QString(dev.c_str ()) + QString (" ") + i18n ("initializing..."));
  } 
}


//void ModemStatus::StartTimerLoop

void ModemStatus::TakeUp ()
{
  if (!is_up) {
    my_status -> Reset ();
    m_manager -> setInterfaceUp (dev, true);
  }
  button -> setPixmap (*online);
  is_up = true;
  if (!m_timer -> isActive ())
    m_timer -> start (500);
}

void ModemStatus::TakeDown ()
{
  if (is_up)
    m_manager -> setInterfaceUp (dev, false);
  button -> setPixmap (*offline);
  is_up = false;
}

void ModemStatus::ButtonPushed ()
{
  if (is_up)
    TakeDown ();
  else
    TakeUp ();
}

QString ModemStatus::TimeToString ()
{
  unsigned int contime = my_status -> GetConnectionTime ();
  unsigned int seconds = contime % 60;
  contime /= 60;
  unsigned int minutes = contime % 60;
  contime /= 60;
  unsigned int hours = contime;
  char buffer[64];
  sprintf(buffer, "%02d:%02d:%02d", hours, minutes, seconds);
  return QString (buffer);
}
