#include "profile_manager.hh"
#include "systeminfo.hh"

#include <klocale.h>
#include <algorithm>
#include <iostream>
#include "dump_config.hh"

ProfileManager::ProfileManager ()
{
  m_manager = new network_config::ConfigManager ();
  ::InitConfig ();
  LoadConfig ();
  DetectEtherDevices (ether_devs);

  for (unsigned int i = 0; i < ether_devs.size (); i++)
    devices.push_back (ether_devs[i].dev);
  devices.push_back ("ppp0");

  std::cout << "available devices :";
  for (unsigned int i = 0; i < devices.size (); i++)
    std::cout << " " << devices[i];
  std::cout << std::endl << "existing profiles :";
  for (unsigned int i = 0; i < profiles.size (); i++)
    std::cout << " " << profiles[i];
  std::cout << std::endl;

  // try to detect active profile
  active_profile_index = -1;
  std::string prof;
  for (unsigned int i=0; i < devices.size() && active_profile_index < 0; i++)
    if (isUp (devices[i], &prof)) {
      active_profile_index = FindIndex (prof);
      if (active_profile_index < 0)
	std::cout << "warning: unknown profile " << prof << " marked as active" << std::endl;
    }

  // if there is no active profile, set config profile to default
  if (active_profile_index != -1)
    config_profile_index = active_profile_index;
  else // we rely on the fact, that default exists (ensured by Loadconfig)
    config_profile_index = std::max(0, FindIndex ("default"));

  ReScanSerials ();

  signal_lock = false;
}

ProfileManager::~ProfileManager ()
{

}


void ProfileManager::LoadConfig ()
{
  profiles.clear ();
  m_manager -> CleanAll ();

  std::ifstream stream;
  if (ConfigfileHandleForReading (stream))
    m_manager -> Parse (stream);
  else
    m_manager -> AppendItem (new kvtree::UnknownLine ("forward"));

  std::vector <profile_id> ids;
  kvtree::Group& root = *(m_manager -> getRoot ());
  for (unsigned int i = 0; i < root.Subnodes (); i++)
    if (root[i] -> Type () == kvtree::node_types::interface_section) {
      network_config::InterfaceSection* section = (network_config::InterfaceSection*) root[i];
      ids.push_back (section -> profile);
    }

  if (ids.size () != 0) {
    std::sort (ids.begin (), ids.end ());
    //    profiles.clear ();
    for (unsigned int i = 0; i < ids.size(); i++)
      if (i == 0 || ids[i] != ids[i-1])
	profiles.push_back (ids[i]);
  } else
    profiles.push_back ("default");

  m_manager -> MoveCurrent (m_manager -> getRoot ());
  stream.close ();
}

void ProfileManager::SaveConfig ()
{
  std::cout << "saving config" << std::endl;
  std::ofstream stream;
  if (! ConfigfileHandleForWriting (stream) ) {
    std::cout << "critical error: could not get stream for configfile !" << std::endl;
    return;
  }
  m_manager -> Dump (stream);
  stream.close ();
}

void ProfileManager::RevertConfig ()
{
  std::cout << "reverting config" << std::endl;
  std::string configure = profiles[config_profile_index];
  std::string active;
  if (active_profile_index >= 0)
    active = profiles[active_profile_index];

  LoadConfig ();

  config_profile_index = FindIndex (configure);
  active_profile_index = FindIndex (active);
  if (config_profile_index < 0)
    config_profile_index = std::min(0, FindIndex ("default"));

  emit newProfileList ();

  emit changedConfigProfile ();
  emit changedActiveProfile ();
}




QString ProfileManager::getI18NProfileName (const profile_id& id)
{
  if (id == "default")
    return i18n ("default");
  else
    return id.c_str();
}

bool operator< (const SerialDev& a, const SerialDev& b)
{
  return a.dev < b.dev;
} 

void ProfileManager::ReScanSerials ()
{
  // first we search for all devices stuff mentioned in the config file
  for (unsigned int i = 0; i < m_manager -> getRoot () -> childs.size(); i++)
    if (m_manager -> getRoot () -> childs [i] -> Type ()== kvtree::node_types::interface_section) {
      network_config::InterfaceSection* section = (network_config::InterfaceSection*) (m_manager -> getRoot () -> childs [i]);
      network_config::PPPSubsection* ppp = section -> GetPPPSubsection (0);
      if (ppp && !(ppp -> OverEthernet ())) {
	network_config::DeviceLine* dl = ppp -> Device();
	if (dl && dl -> device != "")
	  AddSerialUnique (serial_devs, dl -> device);
      }
    }

  // merge with system devs, sort and search for attached physical pnp devices
  DetectSerialDevices (serial_devs);
  if (serial_devs.size() > 0) {
    std::sort (serial_devs.begin(), serial_devs.end());
    ScanSerialDevices (serial_devs);
  } else {
    std::cout << "no serial devices yet, assuming '/dev/tts/0' exists" << std::endl;
    AddSerialUnique (serial_devs, "/dev/tts/0");
  }
  
  // translate device filenames TODO: i18n
  serials.clear ();
  for (unsigned int i = 0; i < serial_devs.size (); i++) {
    const char* dfile = serial_devs[i].dev.c_str();
    QString translated;
    int number;
    if (sscanf(dfile, "/dev/tts/%d", &number) == 1) {
      translated.setNum (number+1);
      translated = i18n("COM") + QString(" ") + translated;
      if (serial_devs[i].attached != "")
	translated += QString(" ") + QString(serial_devs[i].attached.c_str());
    } else if (sscanf(dfile, "/dev/usb/tts/%d", &number) == 1) {
      if (serial_devs[i].attached != "")
	translated += i18n("USB") + QString(" ") + QString(serial_devs[i].attached.c_str());
      else {
	translated.setNum (number+1);
	translated = i18n("USB serial") + QString(" ") + translated;
      }
    } else
      translated = dfile;
    serials.push_back (translated);
  }

  std::cout << "known serials :";
  for (unsigned int i = 0; i < serials.size (); i++)
    std::cout << " '" << serials[i].ascii () << "'";
  std::cout << std::endl;
  emit newSerialList ();
}



EtherDev* ProfileManager::getInfo (unsigned int device_index)
{
  return (device_index >= ether_devs.size ()) ? (EtherDev*) 0 : &(ether_devs [device_index]); 
}

SerialDev* ProfileManager::getSerial (unsigned int serial_index)
{
  return (serial_index >= serial_devs.size ()) ? (SerialDev*) 0 : &(serial_devs [serial_index]); 
}



network_config::InterfaceSection* ProfileManager::getSection (int profile_index, const device_id& device)
{
  if (profile_index < 0 || profile_index >= (int) profiles.size ())
    return 0;

  const profile_id& profile = profiles[profile_index];

  kvtree::Group& root = *(m_manager -> getRoot ());
  for (unsigned int i = 0; i < root.Subnodes (); i++)
    if (root[i] -> Type () == kvtree::node_types::interface_section) {
      network_config::InterfaceSection* section = (network_config::InterfaceSection*) root[i];
      if (section -> profile == profile && section -> interface == device)
	return section;
    }
  
  // requested section does not exist yet, so create it
  // gcc 3.4 compiler error when *root is used instead of &root
  std::cout << "creating " << device << "(" << profile << ")" << std::endl;
  network_config::InterfaceSection* new_section = new network_config::InterfaceSection ( device, profile, &root);
  new_section -> setEnabled (false);
  m_manager -> AppendItem (new_section);
  return new_section;
}

void ProfileManager::ActivateProfile (unsigned int profile_index)
{
  SaveConfig ();

  if (active_profile_index != -1) {
    //    signal_lock = true;
    DeactivateProfile ();
    //  signal_lock = false;
  }
  
  WriteConfig ();

  active_profile_index = profile_index;
  std::cout << "activating " << profiles[active_profile_index] << std::endl;
  emit changedActiveProfile ();
}

void ProfileManager::DeactivateProfile ()
{
  std::cout << "deactivating " << profiles[active_profile_index] << std::endl;
  active_profile_index = -1;

  if (!signal_lock)
      emit changedActiveProfile ();
}

void ProfileManager::ConfigureProfile (unsigned int profile_index)
{
  config_profile_index = profile_index;
  emit changedConfigProfile ();
}


bool ProfileManager::interfaceIsUp (const device_id& device)
{
  std::string profile;
  bool status = isUp (device, &profile);
  if (active_profile_index >= 0 && (profile != profiles[active_profile_index]))
    std::cout << "warning: incompatibe interface section active : " << profile << " ! " << std::endl;
  return status;
}

void ProfileManager::setInterfaceUp (const device_id& device, bool status)
{
  if (!status) {
    std::string prof;
    if (isUp(device, &prof)) {
      std::cout << "taking down " + device << std::endl;
      TriggerIfDown (device, prof);
    }
  } else {
    if (!isUp(device)) {
      std::cout << "seting up " + device << std::endl;
      TriggerIfUp (device, profiles[active_profile_index]);
    }
  }
}

unsigned int ProfileManager::AddProfile (const profile_id& id)
{
  profile_id current_active;
  if (active_profile_index != -1)
    current_active = profiles[active_profile_index];

  profiles.push_back (id);
  std::sort (profiles.begin (), profiles.end ());
  if (active_profile_index != -1)
    active_profile_index = FindIndex (current_active);
  config_profile_index = FindIndex (id);
  emit newProfileList ();
  emit changedActiveProfile ();
  emit changedConfigProfile ();
  return config_profile_index;
}


void ProfileManager::RenameProfile (const profile_id& id_old, const profile_id& id_new)
{
  profile_id current_active;
  if (active_profile_index != -1)
    current_active = profiles[active_profile_index];
  int pos = FindIndex (id_old);
  if (pos < 0)
    return;

  profiles[pos] = id_new;
  std::sort (profiles.begin (), profiles.end ());
  if (active_profile_index != -1)
    active_profile_index = FindIndex (current_active);
  config_profile_index = FindIndex (id_new);

  // rename the InterfaceSections
  kvtree::Group& root = *(m_manager -> getRoot ());
  for (unsigned int i = 0; i < root.Subnodes (); i++)
    if (root[i] -> Type () == kvtree::node_types::interface_section) {
      network_config::InterfaceSection* section = (network_config::InterfaceSection*) root[i];
      if (section -> profile == id_old)
	section -> profile = id_new;
    }

  emit newProfileList ();
  emit changedActiveProfile ();
  emit changedConfigProfile ();
}


void ProfileManager::RemoveProfile (unsigned int profile_index)
{
  profile_id current_active;
  profile_id current_config;

  if ((int) profile_index == active_profile_index)
    DeactivateProfile ();
  else if (active_profile_index != -1)
    current_active = profiles[active_profile_index];
  current_config = profiles[config_profile_index];

  // disable all profile interfaces, but do not delete them yet !
  const profile_id& profile = profiles[profile_index];
  kvtree::Group& root = *(m_manager -> getRoot ());
  for (unsigned int i = 0; i < root.Subnodes (); i++)
    if (root[i] -> Type () == kvtree::node_types::interface_section) {
      network_config::InterfaceSection* section = (network_config::InterfaceSection*) root[i];
      if (section -> profile == profile)
	section -> setEnabled(false);
    }

  profiles[profile_index] = profiles.back ();
  profiles.pop_back ();
  std::sort (profiles.begin (), profiles.end ());
  if (active_profile_index != -1)
    active_profile_index = FindIndex (current_active);

  config_profile_index = FindIndex (current_config);
  if (config_profile_index == -1)
    config_profile_index = 0;

  emit newProfileList ();
  emit changedActiveProfile ();
  emit changedConfigProfile ();
}

void ProfileManager::DebugConfig ()
{
  WalkTree (m_manager -> getRoot ());
}



int ProfileManager::FindIndex (const profile_id& name)
{
  int index = 0;
  while (profiles[index] != name) {
    index++;
    if (index == (int) profiles.size ()) 
      return -1;
  }
  return index;
}

