#ifndef __KVTREE_HH__
#define __KVTREE_HH__

#include <iostream>
#include <vector>
#include <string>

namespace kvtree {

  typedef int node_type;

  namespace node_types {
    // to be extended by user in a separate section ...
    // this is why c++ constants are supperior to dump enums :)
    const node_type global = 0;
    const node_type unknown = -1;
  }


  class kvNode
  {
  public:
    virtual bool isTerminal () = 0;
    virtual node_type Type () = 0;
    virtual void Dump (std::ostream& stream) = 0;
  };


  class Group : public kvNode
  {
  public:
    virtual bool isTerminal () { return false; }

    std::vector <kvNode*> childs;

    Group (Group* parent) { myparent = parent; }

    // !! destructor destructs all childs, so do not call for objects
    // !! which are refered to from the undo chain ! 
    virtual ~Group (); 

    Group* Parent () {return myparent;}

    kvNode* operator[] (unsigned int index) {return childs[index];}
    unsigned int Subnodes () {return childs.size();}
 
    virtual void Dump (std::ostream& stream) {DumpChilds (stream);}

  protected:
    void DumpChilds (std::ostream& stream);

  private:
    Group* myparent;
  };


  class Global : public Group
  {
  public:
    Global () : Group (0) {};
    virtual node_type Type () { return node_types::global; }
  };

  class Line : public kvNode
  {
  public:
    std::string line;

    Line (const std::string& content) { line=content; }
    virtual ~Line () {}

    virtual bool isTerminal () { return true; }
    virtual void Dump (std::ostream& stream) { stream << "\t" << line << std::endl; }
  };


  class UnknownLine : public Line
  {
  public:
    UnknownLine (const std::string& content) : Line (content) {}
    virtual node_type Type () { return node_types::unknown; }
  };


  class TreeManager;

  class LineFactoryInstance
  {
  public:
    // the reference to the content string is _not_ const, so an LF instance
    // is allowed for changing the content.
    virtual bool Match (std::string& content, TreeManager* manager) = 0;
  };


  class TreeManager
  { // Todo: undo / redo classes
  public:
    std::vector <LineFactoryInstance*> lfi;

    void Parse (std::istream& stream);
    void Dump (std::ostream& stream);

    TreeManager (Global* initial_root = new Global());
    virtual ~TreeManager ();

    Group* getRoot () { return root; }
    Group* getCurrent () { return current; }
    
    void MoveCurrent ( Group* new_current) { current = new_current; }

    bool SwitchToParallelSection ( node_type parent, node_type group_type );
    void OpenNewSubsection ( Group* new_group );
    void AppendItem ( kvNode* node );
    
    virtual void ParserWarning (const std::string& warning);

  protected:
    // theese two are usefull for ParserWarning implementations
    unsigned int line_number;
    std::string line_content;

    Group* root;
    Group* current;

  };

} // end namespace kvtree

#endif //  __KVTREE_HH__
