#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <iostream>

#include "ppp_status.hh"


PPPStatus::PPPStatus (const std::string& logfile_name)
{
  log_name = logfile_name;
  old_mod_date = 0;
  old_size = 0;
  connection_start = time (0);
  connected = true;
  last_log_pos = 0;
  search_bound = 0;
  UpdateStatus ();
  if (current != ppp_states::connected && current != ppp_states::connecting)
    connected = false;
}

PPPStatus::~PPPStatus ()
{
}

void PPPStatus::Reset ()
{
  connected = false;
  current = ppp_states::no_connection;
  search_bound = last_log_pos;
}


ppp_state PPPStatus::GetStatus ()
{
  struct stat s;
  if (stat(log_name.c_str(), &s) != 0) {
    std::cerr << "could not stat logfile, assuming offline" << std::endl;
    current = ppp_states::no_connection;
    error = ppp_errors::no_error;
  } else {
    if (s.st_size != old_size || s.st_mtime != old_mod_date) {
      UpdateStatus ();
      old_size = s.st_size;
      old_mod_date = s.st_mtime;
    }
  }
  return current;
}

unsigned int PPPStatus::GetConnectionTime ()
{
  return time (0) - connection_start;
}



bool PPPStatus::KeyWordAction (const std::string& line, const char* kw, ppp_state next, bool setnext, bool n_cmp)
{
  if (line == kw) {
    if (setnext)
      current = next;
    return true;
  }
  return false;
}


bool PPPStatus::KeyWordError (const std::string& line, const char* kw, ppp_error res_error, bool n_cpm)
{
  if (line == kw) {
    current = ppp_states::terminated;
    error = res_error;
    return true;
  }
  return false;
}

void PPPStatus::UpdateStatus ()
{
  bool allready_connected = connected; 
  UpdateStatusInternal ();
  if (current == ppp_states::terminated && !allready_connected)
    current = ppp_states::no_connection;
}

void PPPStatus::UpdateStatusInternal ()
{
  std::cout << "update status" << std::endl;
  current = ppp_states::no_connection;
  error = ppp_errors::no_error;
  ReverseFile rf(log_name, search_bound);
  last_log_pos = rf.FilePos ();

  bool search_error = false;
  bool terminated = false;
  std::cout << "==================== " << last_log_pos << std::endl;
  while (!rf.bof()) {
    std::string line = rf();
    std::cout << "  STATUS  " << rf.FilePos() << " " << line << std::endl;

    if (KeyWordAction (line, "expect (OK)", ppp_states::connecting, (!search_error) && (!terminated))) return;
    //if (KeyWordAction (line, "Using interface ", ppp_states::connecting, !search_error, true)) return;
    if (KeyWordAction (line, "Using interface ppp0", ppp_states::connected, (!search_error) && (!terminated))) {
      if (!connected)
	connection_start = time (0);
      connected = true;
      return;
    }

    if (KeyWordAction (line, "Connection terminated.", ppp_states::terminated, (!search_error) && (!terminated)))
      terminated = true;

    if (KeyWordAction (line, "Connect script failed", ppp_states::terminated, true)) {
      search_error = true;
      error = ppp_errors::unknown;
    }


    if (KeyWordError (line, "BUSY", ppp_errors::busy)) return;
    if (KeyWordError (line, "NO DIALTONE", ppp_errors::no_dialtone)) return;
    if (KeyWordError (line, "LCP terminated by peer", ppp_errors::hangup)) return;
    if (KeyWordError (line, "PAP authentication failed", ppp_errors::authentification)) return;
    if (KeyWordError (line, "CHAP authentication failed", ppp_errors::authentification)) return;

    if (KeyWordError (line, "alarm", ppp_errors::timeout)) return;
  }
}

const unsigned int block_size = 512;

ReverseFile::ReverseFile (const std::string& file_name, unsigned int low_bound)
{
  struct stat s;
  offset = low_bound;
  f = 0;
  suffix = "";
  buffer = new char[block_size + 1]; // +1 needed for the hack in CopyBuf
  next_line_start = 0;

  if (stat(file_name.c_str(), &s) != 0) {
    std::cerr << "could not stat " << file_name << std::endl;
    file_pos = 0;
  } else {
    if (s.st_size < (int) offset) {
      std::cerr << "warining : file " << file_name << " smaller than backward-search bound" << std::endl;
      file_pos = 0;
    } else
      file_pos = s.st_size - offset;
    f = fopen (file_name.c_str(), "r");
    if (!f) {
      std::cerr << "could not open " << file_name << std::endl;
      file_pos = 0;
    }
  }
}

ReverseFile::~ReverseFile ()
{
  if (f)
    fclose (f);
  delete[] buffer; 
}

std::string ReverseFile::CopyBuff (unsigned int lend)
{
  // this is a HACK, but a very good one
  buffer[lend] = '\0';
  return std::string (&(buffer[next_line_start]));
}

std::string ReverseFile::PrevString ()
{
  unsigned int lend = next_line_start;
  do {
    if (next_line_start == 0) {
      suffix = CopyBuff (lend) + suffix;
      if (!bof ()) {
	PrevBuffer ();
	return PrevString ();
      } else
	return suffix;
    }

    next_line_start--;
    if (buffer[next_line_start] == '\n') {
      next_line_start++;
      std::string ret = CopyBuff (lend) + suffix;
      next_line_start--;
      suffix = "";
      return ret;
    } //else
      //  std::cout << next_line_start << " " << buffer[next_line_start] << std::endl;
   
  }  while (true);
}

void ReverseFile::PrevBuffer ()
{
  unsigned int new_file_pos = file_pos - std::min(block_size, file_pos);
  if (fseek(f, new_file_pos + offset, SEEK_SET) != 0) {
    std::cerr << "could not seek in file !" << std::endl;
  }

  int rd_chars = fread ((void*)buffer, 1, file_pos - new_file_pos, f);
  if (rd_chars == 0) {
    std::cerr << "could not read file !" << std::endl;
  }
  //  std::cout << rd_chars << std::endl;
  //getchar ();
  next_line_start = rd_chars;
  file_pos = new_file_pos;
}
